function [u, lambda, logl] = pca1_batch(Y, u_0, lambda_0, nit)

% pca1_batch     Batch EM algorithm for the single factor PCA model.
% Use: [u, lambda, logl] = pca1_batch(Y, u_0, lambda_0, nit)
%       Y: observations (d x n)
%       nit: number of EM iterations
% and
%       u: (d x 1 x nit + 1)
%       lambda: (1 x 1 x nit + 1)
%       logl: (1 x nit)
%
% $Id: pca1_batch.m,v 1.1 2010-07-15 17:08:14 cappe Exp $

% Dimension
[d,n] = size(Y);

% Initialization
u = zeros(d,1,nit+1);
lambda = zeros(1,1,nit+1);
u(:,1,1) = u_0;
lambda(1,1,1) = lambda_0;

for iEM = 1:nit
    S0 = 0;
    S1 = zeros(d,1);
    S2 = 0;
    for iObs = 1:n
        % The E step
        [S0_step, S1_step, S2_step] = pca1_e(Y(:,iObs), u(:,:,iEM), lambda(:,:,iEM));
        S0 = S0 + S0_step;
        S1 = S1 + S1_step;
        S2 = S2 + S2_step;
    end
    [u(:,:,iEM+1), lambda(:,:,iEM+1)] = pca1_m(S0/n, S1/n, S2/n);
    logl(iEM) = -1/2*( n*d*(log(2*pi)+ log(lambda(:,:,iEM))) + ... 
                       (S0 - 2*u(:,:,iEM)'*S1 + S2*(u(:,:,iEM)'*u(:,:,iEM)))/lambda(:,:,iEM) ); 
end
