function [u, lambda] = pca1_online(Y, u_0, lambda_0, gam)

% pca1_online     On-line EM algorithm for the single factor PCA model.
% Use: [u, lambda] = pca1_online(Y, u_0, lambda_0, gam)
%       Y: observations (d x n)
%       gam: step size (1 x n)
% and
%       u: (d x 1 x n + 1)
%       lambda: (1 x 1 x n + 1)
%
% $Id: pca1_online.m,v 1.2 2010-07-15 17:08:14 cappe Exp $

% Parameters
INHIBIT_LAG = 5;
% In inhibition stage we let gamma(i) = 1/i and do not perfom update
gam(1:INHIBIT_LAG) = 1./(1:INHIBIT_LAG);

% Dimension
[d,n] = size(Y);

% Initialization
u = zeros(d,1,n+1);
lambda = zeros(1,1,n+1);
u(:,1,1) = u_0;
lambda(1,1,1) = lambda_0;

S0 = 0;
S1 = zeros(d,1);
S2 = 0;
for i = 1:n
    % The E-SA step
    [S0_step, S1_step, S2_step] = pca1_e(Y(:,i), u(:,:,i), lambda(:,:,i));
    S0 = (1-gam(i))*S0 + gam(i)*S0_step;
    S1 = (1-gam(i))*S1 + gam(i)*S1_step;
    S2 = (1-gam(i))*S2 + gam(i)*S2_step;
    % M-step
    if (i >= INHIBIT_LAG)
      [u(:,:,i+1), lambda(:,:,i+1)] = pca1_m(S0, S1, S2);
    else
      % Inhibition phase
      u(:,:,i+1) = u(:,:,i);
      lambda(:,:,i+1) = lambda(:,:,i);
    end
end
