function [wght, rate, logl] = pm_nealhint(count, wght, rate, Nit);

%pm_em	  Estimates the parameters of a Poisson mixture using the incremental EM algorithm.
%         Use: [wght,rate,logl] = pm_nealhint(count,wght_0,rate_0,Nit,alpha).
%
% $Id: pm_nealhint.m,v 1.2 2010-07-22 17:45:48 cappe Exp $

% Param.
Ninhibit = 5;

% Check input arguments
error(nargchk(4, 4, nargin));
% Data length
T = length(count);
if (any(count < 0) | any(count ~= fix(count)))
  error('Data does not contain positive integers.');
end
count = reshape(count, T, 1);
% Number of mixture components
N = pm_chk(wght, rate);
wght = reshape(wght, 1, N);
rate = reshape(rate, 1, N);

% Variables
logl = zeros(1,Nit);
postprob = zeros(T, N);
% Compute log(count!), the second solution is usually much faster
% except if max(count) is very large
cm = max(count);
if (cm > 50000)
  dnorm = gammaln(count + 1);
else
  tmp = cumsum([0; log((1:max(count)).')]);
  dnorm = tmp(count+1);
end

% Main loop of the EM algorithm
for nit = 1:Nit
  if (nit == 1)
    for nobs = 1:T
      % Replace just one element
      postprob(nobs,:) = exp(-rate + count(nobs)*log(rate) - dnorm(nobs));
      % Compute unormalized a posteriori probability
      postprob(nobs,:) = postprob(nobs,:) .* wght;
      % Normalization
      postprob(nobs,:) = postprob(nobs,:) / sum(postprob(nobs,:));
      if (nobs > Ninhibit)
        %%% 2: Reestimating mixture parameters (M) - THIS ONE IS VERY INEFICIENT
        % Unormalized weights
        wght = sum(postprob(1:nobs,:));
        % Intensities
        rate = (count(1:nobs).' * postprob(1:nobs,:)) ./ wght;
        % Normalize weigths
        wght = wght / sum(wght);          
      end
    end      
  else
    for nobs = 1:T
      % Replace just one element
      postprob(nobs,:) = exp(-rate + count(nobs)*log(rate) - dnorm(nobs));
      % Compute unormalized a posteriori probability
      postprob(nobs,:) = postprob(nobs,:) .* wght;
      % Normalization
      postprob(nobs,:) = postprob(nobs,:) / sum(postprob(nobs,:));
      %%% 2: Reestimating mixture parameters (M) - THIS ONE IS VERY INEFICIENT
      % Unormalized weights
      wght = sum(postprob);
      % Intensities
      rate = (count.' * postprob) ./ wght;
      % Normalize weigths
      wght = wght / sum(wght);
    end
  end
  % Compute all densities - JUST for LL
  postprob_logl = exp(-ones(T,1)*rate + count*log(rate) - dnorm*ones(1,N));
  % Compute unormalized a posteriori probability
  postprob_logl = postprob_logl .* (ones(T,1) * wght);
  % Compute loglikelihood
  logl(nit) = sum(log(sum(postprob_logl')));
  fprintf(1, 'Iteration %d:\t%.3f\n', (nit-1), logl(nit));
end
